@@ -450,6 +450,13 @@ void ClassPathZipEntry::contents_do(void f(const char* name, void* context), voi
   }
 }
 
+void ClassPathImageEntry::close_jimage() {
+  if (_jimage != NULL) {
+    (*JImageClose)(_jimage);
+    _jimage = NULL;
+  }
+}
+
 ClassPathImageEntry::ClassPathImageEntry(JImageFile* jimage, const char* name) :
   ClassPathEntry(),
   _jimage(jimage) {
@@ -731,8 +738,15 @@ void ClassLoader::update_module_path_entry_list(const char *path, TRAPS) {
 void ClassLoader::setup_module_search_path(const char* path, TRAPS) {
   update_module_path_entry_list(path, THREAD);
 }
+
 #endif // INCLUDE_CDS
 
+void ClassLoader::close_jrt_image() {
+  // Not applicable for exploded builds
+  if (!ClassLoader::has_jrt_entry()) return;
+  _jrt_entry->close_jimage();
+}
+
 // Construct the array of module/path pairs as specified to --patch-module
 // for the boot loader to search ahead of the jimage, if the class being
 // loaded is defined to a module that has been specified to --patch-module.
