@@ -346,6 +346,11 @@ void ClassLoaderData::loaded_classes_do(KlassClosure* klass_closure) {
   for (Klass* k = OrderAccess::load_acquire(&_klasses); k != NULL; k = k->next_link()) {
     // Do not filter ArrayKlass oops here...
     if (k->is_array_klass() || (k->is_instance_klass() && InstanceKlass::cast(k)->is_loaded())) {
+#ifdef ASSERT
+      oop m = k->java_mirror();
+      assert(m != NULL, "NULL mirror");
+      assert(m->is_a(SystemDictionary::Class_klass()), "invalid mirror");
+#endif
       klass_closure->do_klass(k);
     }
   }
@@ -658,6 +663,15 @@ Dictionary* ClassLoaderData::create_dictionary() {
   return new Dictionary(this, size, resizable);
 }
 
+void ClassLoaderData::exchange_holders(ClassLoaderData* cld) {
+  oop holder_oop = _holder.peek();
+  _holder.replace(cld->_holder.peek());
+  cld->_holder.replace(holder_oop);
+  WeakHandle<vm_class_loader_data> exchange = _holder;
+  _holder = cld->_holder;
+  cld->_holder = exchange;
+}
+
 // Tell the GC to keep this klass alive while iterating ClassLoaderDataGraph
 oop ClassLoaderData::holder_phantom() const {
   // A klass that was previously considered dead can be looked up in the
@@ -760,6 +774,14 @@ ClassLoaderData::~ClassLoaderData() {
   if (_deallocate_list != NULL) {
     delete _deallocate_list;
   }
+
+  // Decrement refcounts of Symbols if created.
+  if (_name != NULL) {
+    _name->decrement_refcount();
+  }
+  if (_name_and_id != NULL) {
+    _name_and_id->decrement_refcount();
+  }
 }
 
 // Returns true if this class loader data is for the app class loader
@@ -1229,6 +1251,7 @@ void ClassLoaderDataGraph::dictionary_classes_do(void f(InstanceKlass*)) {
   }
 }
 
+// (DCEVM) - iterate over dict classes
 void ClassLoaderDataGraph::dictionary_classes_do(KlassClosure* klass_closure) {
   FOR_ALL_DICTIONARY(cld) {
     cld->dictionary()->classes_do(klass_closure);
@@ -1244,6 +1267,7 @@ void ClassLoaderDataGraph::dictionary_classes_do(void f(InstanceKlass*, TRAPS),
   }
 }
 
+// (DCEVM) rollback redefined classes
 void ClassLoaderDataGraph::rollback_redefinition() {
   FOR_ALL_DICTIONARY(cld) {
     cld->dictionary()->rollback_redefinition();
@@ -1374,12 +1398,13 @@ bool ClassLoaderDataGraph::do_unloading(bool clean_previous_versions) {
   // Klassesoto delete.
 
   // FIXME: dcevm - block asserts in MetadataOnStackMark
-  /*
-  bool walk_all_metadata = clean_previous_versions &&
-                           JvmtiExport::has_redefined_a_class() &&
-                           InstanceKlass::has_previous_versions_and_reset();
-  MetadataOnStackMark md_on_stack(walk_all_metadata);
-  */
+  bool walk_all_metadata = false;
+  if (!AllowEnhancedClassRedefinition) {
+    walk_all_metadata = clean_previous_versions &&
+                             JvmtiExport::has_redefined_a_class() &&
+                             InstanceKlass::has_previous_versions_and_reset();
+    MetadataOnStackMark md_on_stack(walk_all_metadata);
+  }
 
   // Save previous _unloading pointer for CMS which may add to unloading list before
   // purging and we don't want to rewalk the previously unloaded class loader data.
@@ -1389,12 +1414,12 @@ bool ClassLoaderDataGraph::do_unloading(bool clean_previous_versions) {
   while (data != NULL) {
     if (data->is_alive()) {
       // clean metaspace
-      /*
-      if (walk_all_metadata) {
-        data->classes_do(InstanceKlass::purge_previous_versions);
+      if (!AllowEnhancedClassRedefinition) {
+        if (walk_all_metadata) {
+          data->classes_do(InstanceKlass::purge_previous_versions);
+        }
+        data->free_deallocate_list();
       }
-      data->free_deallocate_list();
-      */
       prev = data;
       data = data->next();
       loaders_processed++;
@@ -1433,6 +1458,13 @@ bool ClassLoaderDataGraph::do_unloading(bool clean_previous_versions) {
       if (data->modules_defined()) {
         data->modules()->purge_all_module_reads();
       }
+      // Clean cached pd lists
+      // It's unlikely, but some loaded classes in a dictionary might
+      // point to a protection_domain that has been unloaded.
+      // The dictionary pd_set points at entries in the ProtectionDomainCacheTable.
+      if (data->dictionary() != NULL) {
+        data->dictionary()->clean_cached_protection_domains();
+      }
       data = data->next();
     }
     JFR_ONLY(post_class_unload_events();)
