@@ -50,7 +50,7 @@
 #include "runtime/threadSMR.hpp"
 #include "runtime/vframe.hpp"
 #include "runtime/vframe_hp.hpp"
-#include "runtime/vm_operations.hpp"
+#include "runtime/vmOperations.hpp"
 #include "utilities/exceptions.hpp"
 
 //
@@ -294,8 +294,10 @@ void JvmtiBreakpoint::each_method_version_do(method_action meth_act) {
   Symbol* m_signature = _method->signature();
 
   // (DCEVM) Go through old versions of method
-  for (Method* m = _method->old_version(); m != NULL; m = m->old_version()) {
-    (m->*meth_act)(_bci);
+  if (AllowEnhancedClassRedefinition) {
+    for (Method* m = _method->old_version(); m != NULL; m = m->old_version()) {
+      (m->*meth_act)(_bci);
+    }
   }
 
   // search previous versions if they exist
